"""
Docstring for tests.test_gnss
"""

import pytest
from pygeodl import gnss
import requests
from contextlib import nullcontext as does_not_raise

# @pytest.mark.parametrize(
#     "example_input,expectation",
#     [
#         (3, does_not_raise()),
#         (2, does_not_raise()),
#         (1, does_not_raise()),
#         (0, pytest.raises(ZeroDivisionError)),
#     ],
# )

def test_unavco_request_basic():
    downloader = gnss.unavco()
    df = downloader.request(station="P201", start_date="2020-01-01", end_date="2020-01-02")
    assert not df.empty
    for column in ['Datetime', 'delta N', 'delta E', 'delta U', 'Std Dev N', 'Std Dev E', 'Std Dev U', 'Solution']:
        assert column in df.columns

@pytest.mark.parametrize("station,start_date,end_date,expected", [
    ("P201", "2020-01-01", "2020-01-02", does_not_raise()), #valid request
    ("XXX", "2020-01-01", "2020-01-02", pytest.raises(requests.exceptions.HTTPError)), #incorrect station name
    #("P201", "2020-01-01", "2020-01-02", does_not_raise()),
])
def test_unavco_request(station, start_date, end_date, expected):
    downloader = gnss.unavco()
    with expected:
        df = downloader.request(station=station, start_date=start_date, end_date=end_date)
        assert not df.empty
        for column in ['Datetime', 'delta N', 'delta E', 'delta U', 'Std Dev N', 'Std Dev E', 'Std Dev U', 'Solution']:
            assert column in df.columns





"""
Docstring for pygeodl.gnss

This module provides functionalities for downloading GNSS (Global Navigation Satellite System) data.
"""

import pandas as pd
import requests
import io
import typing

class unavco():
    """
    A class to handle downloading GNSS data from the UNAVCO repository.
    """

    def __init__(self):
        self.base_url = "https://web-services.unavco.org"
        self.expected_format = "text/csv; charset=utf-8"

    def request(self, station: str, start_date: str, end_date: str) -> pd.DataFrame:
        """
        Request GNSS data from the UNAVCO repository.

        An example API request URL:
            https://web-services.unavco.org/gps/data/position/P201/v3?start=2020-01-01&end=2020-01-02
        """
        url = f"{self.base_url}/gps/data/position/{station}/v3?start={start_date}&end={end_date}"
        response = requests.get(url)
        response.raise_for_status()

        if response.headers.get('Content-Type') != self.expected_format:
            raise ValueError("Unexpected response format")

        #TODO parse header data as metadata
        df = pd.read_csv(io.StringIO(response.text), comment='#')
        #df.set_index('Datetime', inplace=True)

        # Remove leading/trailing whitespace from column names
        df.columns = [col.strip() for col in df.columns]

        return df

